import { createApp } from "vue";
import { createPinia } from "pinia";

import Vuex from "vuex";
import Cookies from "js-cookie";

import "normalize.css/normalize.css";

import Element from "element-plus";
//
import mavonEditor from "mavon-editor";
import "mavon-editor/dist/css/index.css";

// 数据字典
import dict from "./components/Dict";

// 权限指令
import permission from "./components/Permission";
// 初始化css
import "./assets/styles/element-variables.scss";
// global css
import "./assets/styles/index.scss";

// 引入vue-quill-editor样式
import "quill/dist/quill.core.css";
import "quill/dist/quill.snow.css";
import "quill/dist/quill.bubble.css";

// 代码高亮
import VueHighlightJS from "vue-highlightjs";
import "highlight.js/styles/atom-one-dark.css";

import "@/assets/font_icon/iconfont.css";

const app = createApp(App);
app.use(Vuex);

import App from "./App.vue";
import router from "./router/routers";
import store from "./store";

app.use(VueHighlightJS);
app.use(mavonEditor);
app.use(permission);
app.use(dict);
app.use(Element, {
  size: Cookies.get("size") || "small", // set element-ui default size
});

app.config.productionTip = false;

app.use(createPinia());
app.use(router);
app.use(store);

app.mount("#app");

import init from "./assets/icons"; // icon
import "./router/index"; // permission control
init(app);
export default app;
