import router from "./routers";
import store from "@/store";
import Config from "@/settings";
import NProgress from "nprogress"; // progress bar
import "nprogress/nprogress.css"; // progress bar style
import { getToken } from "@/utils/auth"; // getToken from cookie
import { buildMenus } from "@/api/system/menu";
import { filterAsyncRouter } from "@/store/modules/permission";

NProgress.configure({ showSpinner: false }); // NProgress Configuration

const whiteList = ["/login"]; // no redirect whitelist

router.beforeEach((to, from, next) => {
  if (to.meta.title) {
    document.title = to.meta.title + " - " + Config.title;
  }
  NProgress.start();
  if (getToken()) {
    // 已登录且要跳转的页面是登录页
    if (to.path === "/login") {
      next({ path: "/" });
      NProgress.done();
    } else {
      if (store.getters.roles.length === 0) {
        // 判断当前用户是否已拉取完user_info信息
        store
          .dispatch("GetInfo")
          .then((res) => {
            // 拉取user_info
            // 动态路由，拉取菜单
            loadMenus(next, to);
          })
          .catch((err) => {
            console.log(err);
            store.dispatch("LogOut").then(() => {
              location.reload(); // 为了重新实例化vue-router对象 避免bug
            });
          });
        // 登录时未拉取 菜单，在此处拉取
      } else if (store.getters.loadMenus) {
        // 修改成false，防止死循环
        store.dispatch("updateLoadMenus").then((res) => {});
        loadMenus(next, to);
      } else {
        next();
      }
    }
  } else {
    /* has no token*/
    if (whiteList.indexOf(to.path) !== -1) {
      // 在免登录白名单，直接进入
      next();
    } else {
      next(`/login?redirect=${to.path}`); // 否则全部重定向到登录页
      NProgress.done();
    }
  }
});

export const loadMenus = (next, to) => {
  buildMenus().then((res) => {
    const asyncRouter = filterAsyncRouter(res);
    asyncRouter.push({
      path: "/:catchAll(.*)",
      redirect: "/404",
      hidden: true,
    });
    store.dispatch("GenerateRoutes", asyncRouter).then(() => {
      // 存储路由
      console.log(router);
      console.log(asyncRouter);
      let routerSync = [
        {
          name: "-",
          path: "/theme",
          hidden: false,
          redirect: "noredirect",
          component: "Layout",
          alwaysShow: true,
          meta: { title: "终端装修", icon: "theme", noCache: true },
          children: [
            {
              name: "Container",
              path: "container",
              hidden: false,
              component: "theme/container/index",
              meta: { title: "商城装修", icon: "theme", noCache: true },
            },
          ],
        },
        {
          name: "商品管理",
          path: "/shop",
          hidden: false,
          redirect: "noredirect",
          component: "Layout",
          alwaysShow: true,
          meta: { title: "商品管理", icon: "shop", noCache: false },
          children: [
            {
              name: "GoodsAdd",
              path: "goodsAdd",
              hidden: true,
              component: "shop/goods/form",
              meta: { title: "商品新增", icon: "anq", noCache: true },
            },
            {
              name: "GoodsEdit",
              path: "goodsEdit/:id",
              hidden: true,
              component: "shop/goods/form",
              meta: { title: "商品修改", icon: "anq", noCache: true },
            },
            {
              name: "Cate",
              path: "cate",
              hidden: false,
              component: "shop/cate/index",
              meta: { title: "商品分类", icon: "icon", noCache: true },
            },
            {
              name: "Goods",
              path: "goods",
              hidden: false,
              component: "shop/goods/tab",
              meta: { title: "管理商品", icon: "develop", noCache: false },
            },
            {
              name: "ProductRule",
              path: "productRule",
              hidden: false,
              component: "shop/storeProductRule/index",
              meta: { title: "商品规格", icon: "mnt", noCache: true },
            },
            {
              name: "ShippingTemplates",
              path: "shippingTemplates",
              hidden: false,
              component: "shop/shippingTemplates/index",
              meta: {
                title: "运费模板",
                icon: "exit-fullscreen",
                noCache: true,
              },
            },
            {
              name: "Viewlog",
              path: "viewlog",
              hidden: false,
              component: "monitor/log/mlog",
              meta: { title: "浏览记录", icon: "log", noCache: true },
            },
            {
              name: "ProductRelation",
              path: "productRelation",
              hidden: false,
              component: "shop/collect/index",
              meta: { title: "商品收藏", icon: "icon", noCache: true },
            },
            {
              name: "FootRelation",
              path: "footRelation",
              hidden: false,
              component: "shop/foot/index",
              meta: { title: "用户足迹", icon: "list", noCache: true },
            },
          ],
        },
        {
          name: "会员管理",
          path: "/member",
          hidden: false,
          redirect: "noredirect",
          component: "Layout",
          alwaysShow: true,
          meta: { title: "会员管理", icon: "peoples", noCache: true },
          children: [
            {
              name: "Member",
              path: "member",
              hidden: false,
              component: "shop/user/index",
              meta: { title: "会员", icon: "peoples", noCache: true },
            },
            {
              name: "Userlevel",
              path: "userlevel",
              hidden: false,
              component: "shop/userlevel/index",
              meta: { title: "会员等级", icon: "dengji", noCache: true },
            },
            {
              name: "Usertask",
              path: "usertask",
              hidden: false,
              component: "shop/usertask/index",
              meta: { title: "等级任务", icon: "task manege", noCache: true },
            },
            {
              name: "Bill",
              path: "bill",
              hidden: false,
              component: "shop/user/bill",
              meta: { title: "用户账单", icon: "list", noCache: true },
            },
            {
              name: "OnlineMember",
              path: "onlinet",
              hidden: false,
              component: "monitor/online/indext",
              meta: { title: "在线会员", icon: "Steve-Jobs", noCache: true },
            },
          ],
        },
        {
          name: "订单管理",
          path: "/order",
          hidden: false,
          redirect: "noredirect",
          component: "Layout",
          alwaysShow: true,
          meta: { title: "订单管理", icon: "lock", noCache: true },
          children: [
            {
              name: "Order",
              path: "order",
              hidden: false,
              component: "shop/order/index",
              meta: { title: "订单", icon: "order", noCache: true },
            },
            {
              name: "Reply",
              path: "reply",
              hidden: false,
              component: "shop/reply/index",
              meta: { title: "评论管理", icon: "comment", noCache: true },
            },
            {
              name: "Express",
              path: "express",
              hidden: false,
              component: "shop/express/index",
              meta: { title: "物流快递", icon: "express", noCache: true },
            },
            {
              name: "AfterSeals",
              path: "afterSeals",
              hidden: false,
              component: "shop/afterSeals/index",
              meta: { title: "售后", icon: "order", noCache: true },
            },
            {
              name: "Detail",
              path: "detail/:id",
              hidden: true,
              component: "shop/order/detail",
              meta: { title: "订单详情", icon: "sqlMonitor", noCache: true },
            },
          ],
        },
        {
          name: "电子券管理",
          path: "/syscoupon",
          hidden: false,
          redirect: "noredirect",
          component: "Layout",
          alwaysShow: true,
          meta: { title: "电子券管理", icon: "coupon", noCache: true },
          children: [
            {
              name: "Coupon",
              path: "coupon",
              hidden: false,
              component: "activity/coupon/index",
              meta: { title: "优惠券制作", icon: "coupon", noCache: true },
            },
            {
              name: "Couponissue",
              path: "couponissue",
              hidden: false,
              component: "activity/couponissue/index",
              meta: { title: "已发布优惠券", icon: "coupon2", noCache: true },
            },
            {
              name: "Couponuser",
              path: "couponuser",
              hidden: false,
              component: "activity/storecouponuser/index",
              meta: { title: "优惠券领取记录", icon: "log", noCache: true },
            },
          ],
        },
        {
          name: "营销管理",
          path: "/activity",
          hidden: false,
          redirect: "noredirect",
          component: "Layout",
          alwaysShow: true,
          meta: { title: "营销管理", icon: "yingxiao", noCache: true },
          children: [
            {
              name: "CombinationEdit",
              path: "combinationEdit/:id",
              hidden: true,
              component: "activity/combination/form",
              meta: { title: "拼团商品修改", icon: "anq", noCache: true },
            },
            {
              name: "SecKillEdit",
              path: "secKillEdit/:id",
              hidden: true,
              component: "activity/seckill/form",
              meta: { title: "秒杀商品修改", icon: "anq", noCache: true },
            },
            {
              name: "Combination",
              path: "combination",
              hidden: false,
              component: "activity/combination/index",
              meta: { title: "拼团产品", icon: "peoples", noCache: true },
            },
            {
              name: "Pink",
              path: "pink",
              hidden: false,
              component: "activity/combinlist/index",
              meta: { title: "拼团列表", icon: "list", noCache: true },
            },
            {
              name: "Seckill",
              path: "seckill",
              hidden: false,
              component: "activity/seckill/index",
              meta: { title: "秒杀产品", icon: "seckill", noCache: true },
            },
            {
              name: "Seckillconfig",
              path: "seckillconfig",
              hidden: false,
              component: "shop/set/seckill",
              meta: { title: "秒杀配置", icon: "configure", noCache: true },
            },
            {
              name: "Bargain",
              path: "bargain",
              hidden: false,
              component: "activity/bargain/index",
              meta: { title: "砍价产品", icon: "Sign", noCache: true },
            },
            {
              name: "CombinationAdd",
              path: "combinationAdd",
              hidden: true,
              component: "activity/combination/form",
              meta: { title: "拼团商品添加", icon: null, noCache: true },
            },
            {
              name: "SecKillAdd",
              path: "secKillAdd",
              hidden: true,
              component: "activity/seckill/form",
              meta: { title: "秒杀商品添加", icon: null, noCache: true },
            },
          ],
        },
        {
          name: "分销管理",
          path: "/promoter",
          hidden: false,
          redirect: "noredirect",
          component: "Layout",
          alwaysShow: true,
          meta: { title: "分销管理", icon: "fenxiao", noCache: true },
          children: [
            {
              name: "Promoterconfig",
              path: "promoterconfig",
              hidden: false,
              component: "wechat/config/promoter",
              meta: { title: "分销配置", icon: "configure", noCache: true },
            },
            {
              name: "Prolog",
              path: "prolog",
              hidden: false,
              component: "shop/user/brobill",
              meta: { title: "分销记录", icon: "log", noCache: true },
            },
          ],
        },
        {
          name: "财务管理",
          path: "/price",
          hidden: false,
          redirect: "noredirect",
          component: "Layout",
          alwaysShow: true,
          meta: { title: "财务管理", icon: "price", noCache: true },
          children: [
            {
              name: "Extract",
              path: "extract",
              hidden: false,
              component: "activity/extract/index",
              meta: { title: "提现管理", icon: "tixian", noCache: true },
            },
            {
              name: "Recharge",
              path: "recharge",
              hidden: false,
              component: "shop/recharge/index",
              meta: { title: "充值管理", icon: "rec", noCache: true },
            },
            {
              name: "Rechargeset",
              path: "rechargeset",
              hidden: false,
              component: "shop/set/recharge",
              meta: { title: "充值金额配置", icon: "money", noCache: true },
            },
          ],
        },
        {
          name: "门店管理",
          path: "/store",
          hidden: false,
          redirect: "noredirect",
          component: "Layout",
          alwaysShow: true,
          meta: { title: "门店管理", icon: "store", noCache: true },
          children: [
            {
              name: "Storeset",
              path: "storeset",
              hidden: false,
              component: "shop/store/set",
              meta: { title: "门店配置", icon: "configure", noCache: true },
            },
            {
              name: "Storeinfo",
              path: "storeinfo",
              hidden: false,
              component: "shop/store/index",
              meta: { title: "门店列表", icon: "edit", noCache: true },
            },
            {
              name: "Staff",
              path: "staff",
              hidden: false,
              component: "shop/storestaff/index",
              meta: { title: "店员列表", icon: "peoples", noCache: true },
            },
            {
              name: "Ordert",
              path: "ordert",
              hidden: false,
              component: "shop/order/indext",
              meta: { title: "核销订单", icon: "order", noCache: true },
            },
          ],
        },
        {
          name: "-",
          path: "/syspoints",
          hidden: false,
          redirect: "noredirect",
          component: "Layout",
          alwaysShow: true,
          meta: { title: "积分管理", icon: "Sign", noCache: true },
          children: [
            {
              name: "PointConfig",
              path: "pointConfig",
              hidden: false,
              component: "wechat/config/point",
              meta: { title: "积分配置", icon: "configure", noCache: true },
            },
            {
              name: "Pobill",
              path: "pobill",
              hidden: false,
              component: "shop/user/pobill",
              meta: { title: "积分记录", icon: "log", noCache: true },
            },
            {
              name: "Signday",
              path: "signday",
              hidden: false,
              component: "shop/set/sign",
              meta: { title: "签到天数配置", icon: "sign2", noCache: true },
            },
          ],
        },
        {
          name: "微信管理",
          path: "/wechat",
          hidden: false,
          redirect: "noredirect",
          component: "Layout",
          alwaysShow: true,
          meta: { title: "微信管理", icon: "weixin", noCache: true },
          children: [
            {
              name: "WeMenu",
              path: "wemenu",
              hidden: false,
              component: "wechat/menu/index",
              meta: { title: "微信菜单", icon: "menu", noCache: true },
            },
            {
              name: "WeArticle",
              path: "wearticle",
              hidden: false,
              component: "wechat/article/index",
              meta: { title: "图文管理", icon: "article", noCache: true },
            },
            {
              name: "Wereply",
              path: "wereply",
              hidden: false,
              component: "wechat/reply/index",
              meta: { title: "自动回复", icon: "reply", noCache: true },
            },
            {
              name: "WeConfig",
              path: "weconfig",
              hidden: false,
              component: "wechat/config/index",
              meta: { title: "公众号配置", icon: "configure", noCache: true },
            },
            {
              name: "Wxpayconfig",
              path: "wxpayconfig",
              hidden: false,
              component: "wechat/config/pay",
              meta: { title: "微信支付配置", icon: "wxpay", noCache: true },
            },
            {
              name: "Template",
              path: "template",
              hidden: false,
              component: "wechat/template/index",
              meta: { title: "微信模板消息", icon: "anq", noCache: true },
            },
            {
              name: "Wxapp",
              path: "wxapp",
              hidden: false,
              component: "wechat/config/wxapp",
              meta: { title: "小程序配置", icon: "configure", noCache: true },
            },
            {
              name: "Artadd",
              path: "artadd",
              hidden: true,
              component: "wechat/article/form",
              meta: { title: "图文添加", icon: "article", noCache: true },
            },
            {
              name: "Editadd",
              path: "artadd/:id",
              hidden: true,
              component: "wechat/article/form",
              meta: { title: "图文编辑", icon: "alipay", noCache: true },
            },
            {
              name: "Wxlive",
              path: "wxlive",
              hidden: false,
              component: "wechat/live/index",
              meta: { title: "直播管理", icon: "live", noCache: true },
            },
            {
              name: "WxliveGoods",
              path: "wxlivegoods",
              hidden: false,
              component: "wechat/goods/index",
              meta: { title: "直播商品管理", icon: "shopping", noCache: true },
            },
          ],
        },
        {
          name: "商城配置",
          path: "/set",
          hidden: false,
          redirect: "noredirect",
          component: "Layout",
          alwaysShow: true,
          meta: { title: "商城配置", icon: "configure", noCache: true },
          children: [
            {
              name: "msgInfo",
              path: "storeCustomer",
              hidden: false,
              component: "shop/customer/index",
              meta: { title: "订单消息管理", icon: "peoples", noCache: true },
            },
            {
              name: "HomeBanner",
              path: "homeBanner",
              hidden: true,
              component: "shop/set/index",
              meta: { title: "首页幻灯片", icon: "banner", noCache: true },
            },
            {
              name: "HomeMenus",
              path: "homeMenus",
              hidden: true,
              component: "shop/set/menu",
              meta: { title: "首页导航按钮", icon: "button", noCache: true },
            },
            {
              name: "Roll",
              path: "roll",
              hidden: true,
              component: "shop/set/roll",
              meta: { title: "首页滚动新闻", icon: "news", noCache: true },
            },
            {
              name: "Hot",
              path: "hot",
              hidden: false,
              component: "shop/set/hot",
              meta: { title: "热门搜索", icon: "search", noCache: true },
            },
            {
              name: "UserMenu",
              path: "userMenu",
              hidden: false,
              component: "shop/set/usermenu",
              meta: { title: "个人中心菜单", icon: "menu", noCache: true },
            },
            {
              name: "Shopconfig",
              path: "shopconfig",
              hidden: false,
              component: "wechat/config/shop",
              meta: { title: "商城系统配置", icon: "shop", noCache: true },
            },
            {
              name: "Smsconfig",
              path: "smsconfig",
              hidden: false,
              component: "wechat/config/sms",
              meta: { title: "阿里云短信", icon: "sys-tools", noCache: true },
            },
            {
              name: "Expressconfig",
              path: "expressconfig",
              hidden: false,
              component: "wechat/config/express",
              meta: { title: "快递鸟配置", icon: "news", noCache: true },
            },
            {
              name: "App版本",
              path: "appVersion",
              hidden: false,
              component: "shop/appVersion/index",
              meta: { title: "App版本", icon: "app", noCache: true },
            },
          ],
        },
        {
          name: "系统管理",
          path: "/system",
          hidden: false,
          redirect: "noredirect",
          component: "Layout",
          alwaysShow: true,
          meta: { title: "系统管理", icon: "system", noCache: true },
          children: [
            {
              name: "User",
              path: "user",
              hidden: false,
              component: "system/user/index",
              meta: { title: "用户管理", icon: "peoples", noCache: true },
            },
            {
              name: "Role",
              path: "role",
              hidden: false,
              component: "system/role/index",
              meta: { title: "角色管理", icon: "role", noCache: true },
            },
            {
              name: "Menu",
              path: "menu",
              hidden: false,
              component: "system/menu/index",
              meta: { title: "菜单管理", icon: "menu", noCache: true },
            },
            {
              name: "Dept",
              path: "dept",
              hidden: false,
              component: "system/dept/index",
              meta: { title: "部门管理", icon: "dept", noCache: true },
            },
            {
              name: "Job",
              path: "job",
              hidden: false,
              component: "system/job/index",
              meta: { title: "岗位管理", icon: "Steve-Jobs", noCache: true },
            },
            {
              name: "Dict",
              path: "dict",
              hidden: false,
              component: "system/dict/index",
              meta: { title: "字典管理", icon: "dictionary", noCache: true },
            },
          ],
        },
        {
          name: "系统监控",
          path: "/monitor",
          hidden: false,
          redirect: "noredirect",
          component: "Layout",
          alwaysShow: true,
          meta: { title: "系统监控", icon: "monitor", noCache: true },
          children: [
            {
              name: "OnlineUser",
              path: "online",
              hidden: false,
              component: "monitor/online/index",
              meta: { title: "在线用户", icon: "Steve-Jobs", noCache: true },
            },
            {
              name: "Log",
              path: "logs",
              hidden: false,
              component: "monitor/log/index",
              meta: { title: "操作日志", icon: "log", noCache: false },
            },
            {
              name: "ErrorLog",
              path: "errorLog",
              hidden: false,
              component: "monitor/log/errorLog",
              meta: { title: "异常日志", icon: "error", noCache: true },
            },
            {
              name: "Sql",
              path: "druid",
              hidden: false,
              component: "monitor/sql/index",
              meta: { title: "SQL监控", icon: "sqlMonitor", noCache: true },
            },
          ],
        },
        {
          name: "系统工具",
          path: "/sys-tools",
          hidden: false,
          redirect: "noredirect",
          component: "Layout",
          alwaysShow: true,
          meta: { title: "系统工具", icon: "sys-tools", noCache: true },
          children: [
            {
              name: "Timing",
              path: "timing",
              hidden: false,
              component: "system/timing/index",
              meta: { title: "定时任务", icon: "timing", noCache: true },
            },
            {
              name: "GeneratorIndex",
              path: "generator",
              hidden: false,
              component: "generator/index",
              meta: { title: "代码生成", icon: "dev", noCache: true },
            },
            {
              name: "Storage",
              path: "storage",
              hidden: false,
              component: "tools/storage/index",
              meta: { title: "存储管理", icon: "qiniu", noCache: true },
            },
            {
              name: "Email",
              path: "email",
              hidden: false,
              component: "tools/email/index",
              meta: { title: "邮件工具", icon: "email", noCache: true },
            },
            {
              name: "AliPay",
              path: "aliPay",
              hidden: false,
              component: "tools/aliPay/index",
              meta: { title: "支付宝工具", icon: "alipay", noCache: true },
            },
            {
              name: "Swagger",
              path: "swagger2",
              hidden: false,
              component: "tools/swagger/index",
              meta: { title: "后台接口文档", icon: "swagger", noCache: true },
            },
            {
              name: "GeneratorConfig",
              path: "generator/config/:tableName",
              hidden: true,
              component: "generator/config",
              meta: { title: "生成配置", icon: "dev", noCache: false },
            },
            {
              name: "GeneratorConfig",
              path: "generator/config/:tableName",
              hidden: true,
              component: "generator/config",
              meta: { title: "生成配置", icon: "dev", noCache: false },
            },
            {
              name: "Preview",
              path: "generator/preview/:tableName",
              hidden: true,
              component: "generator/preview",
              meta: { title: "生成预览", icon: "java", noCache: false },
            },
          ],
        },
      ];
      asyncRouter.forEach((item) => {
        console.log(item, 222222);
        router.addRoute(item); // 动态添加可访问路由表
      });

      // router.addRoute({
      //   path: "/:catchAll(.*)", // 不识别的path自动匹配404
      //   redirect: "/404",
      // });

      next({ ...to, replace: true });
    });
  });
};

router.afterEach(() => {
  NProgress.done(); // finish progress bar
});
