import { defineAsyncComponent } from "vue";
import { createRouter, createWebHistory } from "vue-router";
import Layout from "../layout/index.vue";

export const constantRouterMap = [
  {
    path: "/login",
    meta: { title: "登录", noCache: true },
    component: defineAsyncComponent(() => import("@/views/login")),
    hidden: true,
  },
  {
    path: "/:pathMatch(.*)",
    name: "404",
    component: defineAsyncComponent(() => import("@/views/features/404")),
    hidden: true,
  },
  {
    path: "/401",
    component: defineAsyncComponent(() => import("@/views/features/401")),
    hidden: true,
  },
  {
    path: "/redirect",
    component: Layout,
    hidden: true,
    children: [
      {
        path: "/redirect/:path*",
        component: defineAsyncComponent(() =>
          import("@/views/features/redirect")
        ),
      },
    ],
  },
  {
    path: "/",
    component: Layout,
    redirect: "/dashboard",
    children: [
      {
        path: "dashboard",
        component: defineAsyncComponent(() => import("@/views/home")),
        name: "Dashboard",
        meta: { title: "首页", icon: "index", affix: true, noCache: true },
      },
    ],
  },
  {
    path: "/user",
    component: Layout,
    hidden: true,
    redirect: "noredirect",
    children: [
      {
        path: "center",
        component: defineAsyncComponent(() =>
          import("@/views/system/user/center")
        ),
        name: "个人中心",
        meta: { title: "个人中心" },
      },
    ],
  },
  {
    path: "/yshop/materia/index",
    name: `images`,
    meta: {
      title: "上传图片",
      noCache: true,
    },
    component: defineAsyncComponent(() =>
      import("@/components/editorMaterial/index")
    ),
    hidden: true,
  },
];

const router = createRouter({
  history: createWebHistory(),
  scrollBehavior: () => ({ y: 0 }),
  routes: constantRouterMap,
});

export default router;
